@@ -1,6 +1,6 @@
 /*
-    Copyright (c) 2014-2018 Christian Schoenebeck
-    
+    Copyright (c) 2014-2020 Christian Schoenebeck
+
     This file is part of "gigedit" and released under the terms of the
     GNU General Public License version 2.
 */
@@ -49,6 +49,7 @@ static std::string groupName(Settings::Group_t group) {
         case Settings::MIDI_RULES: return "MidiRules";
         case Settings::FILE_PROPS: return "FileProps";
         case Settings::INSTR_PROPS: return "InstrProps";
+        case Settings::SAMPLE_PROPS: return "SampleProps";
         case Settings::SAMPLE_REFS: return "SampleRefs";
         case Settings::MACRO_EDITOR: return "MacroEditor";
         case Settings::MACROS_SETUP: return "MacrosSetup";
@@ -87,10 +88,12 @@ Settings::Settings() : Glib::ObjectBase(typeid(Settings)),
     autoRestoreWindowDimension(*this, GLOBAL, "autoRestoreWindowDimension", false),
     saveWithTemporaryFile(*this, GLOBAL, "saveWithTemporaryFile", false),
     showTooltips(*this, GLOBAL, "showNewbieTooltips", true),
+    instrumentDoubleClickOpensProps(*this, GLOBAL, "openInstrPropsByDoubleClick", true),
     mainWindowX(*this, MAIN_WINDOW, "x", -1),
     mainWindowY(*this, MAIN_WINDOW, "y", -1),
     mainWindowW(*this, MAIN_WINDOW, "w", -1),
     mainWindowH(*this, MAIN_WINDOW, "h", -1),
+    mainWindowSplitterPosX(*this, MAIN_WINDOW, "splitterPosX", -1),
     scriptEditorWindowX(*this, SCRIPT_EDITOR, "x", -1),
     scriptEditorWindowY(*this, SCRIPT_EDITOR, "y", -1),
     scriptEditorWindowW(*this, SCRIPT_EDITOR, "w", -1),
@@ -120,6 +123,10 @@ Settings::Settings() : Glib::ObjectBase(typeid(Settings)),
     instrPropsWindowY(*this, INSTR_PROPS, "y", -1),
     instrPropsWindowW(*this, INSTR_PROPS, "w", -1),
     instrPropsWindowH(*this, INSTR_PROPS, "h", -1),
+    samplePropsWindowX(*this, SAMPLE_PROPS, "x", -1),
+    samplePropsWindowY(*this, SAMPLE_PROPS, "y", -1),
+    samplePropsWindowW(*this, SAMPLE_PROPS, "w", -1),
+    samplePropsWindowH(*this, SAMPLE_PROPS, "h", -1),
     sampleRefsWindowX(*this, SAMPLE_REFS, "x", -1),
     sampleRefsWindowY(*this, SAMPLE_REFS, "y", -1),
     sampleRefsWindowW(*this, SAMPLE_REFS, "w", -1),
@@ -140,10 +147,12 @@ Settings::Settings() : Glib::ObjectBase(typeid(Settings)),
     m_boolProps.push_back(&autoRestoreWindowDimension);
     m_boolProps.push_back(&saveWithTemporaryFile);
     m_boolProps.push_back(&showTooltips);
+    m_boolProps.push_back(&instrumentDoubleClickOpensProps);
     m_intProps.push_back(&mainWindowX);
     m_intProps.push_back(&mainWindowY);
     m_intProps.push_back(&mainWindowW);
     m_intProps.push_back(&mainWindowH);
+    m_intProps.push_back(&mainWindowSplitterPosX);
     m_intProps.push_back(&scriptEditorWindowX);
     m_intProps.push_back(&scriptEditorWindowY);
     m_intProps.push_back(&scriptEditorWindowW);
@@ -173,6 +182,10 @@ Settings::Settings() : Glib::ObjectBase(typeid(Settings)),
     m_intProps.push_back(&instrPropsWindowY);
     m_intProps.push_back(&instrPropsWindowW);
     m_intProps.push_back(&instrPropsWindowH);
+    m_intProps.push_back(&samplePropsWindowX);
+    m_intProps.push_back(&samplePropsWindowY);
+    m_intProps.push_back(&samplePropsWindowW);
+    m_intProps.push_back(&samplePropsWindowH);
     m_intProps.push_back(&sampleRefsWindowX);
     m_intProps.push_back(&sampleRefsWindowY);
     m_intProps.push_back(&sampleRefsWindowW);
