@@ -39,9 +39,6 @@ VelocityCurve::VelocityCurve(double (gig::DimensionRegion::*getter)(uint8_t)) :
 bool VelocityCurve::on_expose_event(GdkEventExpose* e) {
     const Cairo::RefPtr<Cairo::Context>& cr =
         get_window()->create_cairo_context();
-#if 0
-}
-#endif
 #else
 bool VelocityCurve::on_draw(const Cairo::RefPtr<Cairo::Context>& cr) {
 #endif
@@ -77,16 +74,13 @@ bool VelocityCurve::on_draw(const Cairo::RefPtr<Cairo::Context>& cr) {
 
 
 CrossfadeCurve::CrossfadeCurve() : dimreg(0) {
-    set_size_request(280, 80);
+    set_size_request(500, 100);
 }
 
 #if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
 bool CrossfadeCurve::on_expose_event(GdkEventExpose* e) {
     const Cairo::RefPtr<Cairo::Context>& cr =
         get_window()->create_cairo_context();
-#if 0
-}
-#endif
 #else
 bool CrossfadeCurve::on_draw(const Cairo::RefPtr<Cairo::Context>& cr) {
 #endif
@@ -155,6 +149,146 @@ void CrossfadeCurve::draw_one_curve(const Cairo::RefPtr<Cairo::Context>& cr,
 }
 
 
+LFOGraph::LFOGraph() : dimreg(0) {
+    set_size_request(500, 100);
+}
+
+#if (GTKMM_MAJOR_VERSION == 2 && GTKMM_MINOR_VERSION < 90) || GTKMM_MAJOR_VERSION < 2
+bool LFOGraph::on_expose_event(GdkEventExpose* e) {
+    const Cairo::RefPtr<Cairo::Context>& cr =
+        get_window()->create_cairo_context();
+#else
+bool LFOGraph::on_draw(const Cairo::RefPtr<Cairo::Context>& cr) {
+#endif
+    if (dimreg) {
+        const int w = get_width();
+        const int h = get_height();
+        const bool sensitive = is_sensitive();
+        const bool signedRange = this->signedRange();
+        const float visiblePeriods = 5.f; // such that minimum LFO frequency 0.1 Hz draws exactly a half period
+
+        // short-hand functions for setting colors
+        auto setGrayColor = [&] {
+            cr->set_source_rgba(0.88, 0.88, 0.88, sensitive ? 1.0 : 0.3);
+        };
+        auto setBlackColor = [&] {
+            cr->set_source_rgba(0, 0, 0, sensitive ? 1.0 : 0.3);
+        };
+        auto setGreenColor = [&] {
+            cr->set_source_rgba(94/255.f, 219/255.f, 80/255.f, sensitive ? 1.0 : 0.3);
+        };
+        auto setRedColor = [&] {
+            cr->set_source_rgba(255.f, 44/255.f, 44/255.f, sensitive ? 1.0 : 0.3);
+        };
+        /*auto setBlueColor = [&] {
+            cr->set_source_rgba(53/255.f, 167/255.f, 255.f, sensitive ? 1.0 : 0.3);
+        };*/
+        auto setOrangeColor = [&] {
+            cr->set_source_rgba(255.f, 177/255.f, 82/255.f, sensitive ? 1.0 : 0.3);
+        };
+        auto setWhiteColor = [&] {
+            cr->set_source_rgba(255.f, 255.f, 255.f, sensitive ? 1.0 : 0.3);
+        };
+
+        // fill background white
+        cr->rectangle(0, 0, w, h);
+        setWhiteColor();
+        cr->fill();
+
+        // draw horizontal center line (dashed gray) if LFO range is signed
+        if (signedRange) {
+            cr->move_to(0, h/2);
+            cr->line_to(w, h/2);
+            cr->set_line_width(2);
+            setGrayColor();
+            cr->set_dash(std::vector<double>{ 7, 5 }, 0 /*offset*/);
+            cr->stroke();
+        }
+
+        // draw a vertical line for each second
+        for (int period = 1; period < visiblePeriods; ++period) {
+            int x = float(w) / float(visiblePeriods) * period;
+            cr->move_to(x, 0);
+            cr->line_to(x, h);
+            cr->set_line_width(2);
+            setGrayColor();
+            cr->set_dash(std::vector<double>{ 5, 3 }, 0 /*offset*/);
+            cr->stroke();
+        }
+
+        // how many curves shall we draw, two or one?
+        const int runs = (hasControllerAssigned()) ? 2 : 1;
+        // only draw the two curves in dashed style if they're very close to each other
+        const bool dashedCurves = (runs == 2 && controllerDepth() < 63);
+        // draw the required amount of curves
+        for (int run = 0; run < runs; ++run) {
+            // setup the LFO generator with the relevant parameters
+            lfo.setup({
+                .waveType = waveType(),
+                .rangeType = (signedRange) ? LinuxSampler::LFO::range_signed : LinuxSampler::LFO::range_unsigned,
+                .frequency = frequency(),
+                .phase = phase(),
+                .startLevel = startLevel(),
+                .internalDepth = internalDepth(),
+                .midiControllerDepth = controllerDepth(),
+                .flipPolarity = flipPolarity(),
+                .samplerate = w / visiblePeriods,
+                .maxValue = (signedRange) ? h/2 : h,
+            });
+            // 1st curve reflects min. CC value, 2nd curve max. CC value
+            lfo.setMIDICtrlValue( (run == 0) ? 0 : 127 );
+
+            // the actual render/draw loop
+            for (int x = 0; x < w ; ++x) {
+                const float y =
+                    (signedRange) ?
+                        h/2 - lfo.render() :
+                        h   - lfo.render();
+                if (x == 0)
+                    cr->move_to(x, y);
+                else
+                    cr->line_to(x, y);
+            }
+            cr->set_line_width( (frequency() <= 4.f) ? 2 : 1 );
+            if (runs == 1)
+                setOrangeColor();
+            else if (run == 0)
+                setGreenColor();
+            else
+                setRedColor();
+            if (dashedCurves)
+                cr->set_dash(std::vector<double>{ 3, 3 }, (run == 0) ? 0 : 3 /*offset*/);
+            else
+                cr->set_dash(std::vector<double>(), 0 /*offset*/);
+            cr->stroke();
+        }
+
+        // draw text legend
+        if (runs == 2) {
+            setRedColor();
+            cr->move_to(2, 10);
+            cr->show_text("CC Max.");
+
+            setGreenColor();
+            cr->move_to(2, 23);
+            cr->show_text("CC Min.");
+        } else { // no controller assigned, internal depth only ...
+            setOrangeColor();
+            cr->move_to(2, 10);
+            cr->show_text("Const. Depth");
+        }
+        // draw text legend for each second ("1s", "2s", ...)
+        for (int period = 1; period < visiblePeriods; ++period) {
+            int x = float(w) / float(visiblePeriods) * period;
+            setBlackColor();
+            cr->move_to(x - 13, h - 3);
+            cr->show_text(ToString(period) + "s");
+        }
+    }
+    return true;
+}
+
+
 EGStateOptions::EGStateOptions() : HBox(),
     label(_("May be cancelled: ")),
     checkBoxAttack(_("Attack")),
@@ -217,7 +351,9 @@ DimRegionEdit::DimRegionEdit() :
     eEG1ControllerAttackInfluence(_("Controller attack influence"), 0, 3),
     eEG1ControllerDecayInfluence(_("Controller decay influence"), 0, 3),
     eEG1ControllerReleaseInfluence(_("Controller release influence"), 0, 3),
+    eLFO1Wave(_("Wave Form")),
     eLFO1Frequency(_("Frequency"), 0.1, 10, 2),
+    eLFO1Phase(_("Phase"), 0.0, 360.0, 2),
     eLFO1InternalDepth(_("Internal depth"), 0, 1200),
     eLFO1ControlDepth(_("Control depth"), 0, 1200),
     eLFO1Controller(_("Controller")),
@@ -235,7 +371,9 @@ DimRegionEdit::DimRegionEdit() :
     eEG2ControllerAttackInfluence(_("Controller attack influence"), 0, 3),
     eEG2ControllerDecayInfluence(_("Controller decay influence"), 0, 3),
     eEG2ControllerReleaseInfluence(_("Controller release influence"), 0, 3),
+    eLFO2Wave(_("Wave Form")),
     eLFO2Frequency(_("Frequency"), 0.1, 10, 2),
+    eLFO2Phase(_("Phase"), 0.0, 360.0, 2),
     eLFO2InternalDepth(_("Internal depth"), 0, 1200),
     eLFO2ControlDepth(_("Control depth"), 0, 1200),
     eLFO2Controller(_("Controller")),
@@ -243,10 +381,13 @@ DimRegionEdit::DimRegionEdit() :
     eLFO2Sync(_("Sync")),
     eEG3Attack(_("Attack"), 0, 10, 3),
     eEG3Depth(_("Depth"), -1200, 1200),
+    eLFO3Wave(_("Wave Form")),
     eLFO3Frequency(_("Frequency"), 0.1, 10, 2),
+    eLFO3Phase(_("Phase"), 0.0, 360.0, 2),
     eLFO3InternalDepth(_("Internal depth"), 0, 1200),
     eLFO3ControlDepth(_("Control depth"), 0, 1200),
     eLFO3Controller(_("Controller")),
+    eLFO3FlipPhase(_("Flip phase")),
     eLFO3Sync(_("Sync")),
     eVCFEnabled(_("Enabled")),
     eVCFType(_("Type")),
@@ -290,8 +431,7 @@ DimRegionEdit::DimRegionEdit() :
     eSampleID("Sample ID"),
     eChecksum("Wave Data CRC-32"),
     eFineTune(_("Fine tune"), -49, 50),
-    eGain(_("Gain"), -96, 0, 2, -655360),
-    eGainPlus6(_("Gain +6dB"), eGain, 6 * -655360),
+    eGain(_("Gain (dB)"), -96, +96, 2, -655360),
     eSampleLoopEnabled(_("Enabled")),
     eSampleLoopStart(_("Loop start position")),
     eSampleLoopLength(_("Loop size")),
@@ -299,12 +439,26 @@ DimRegionEdit::DimRegionEdit() :
     eSampleLoopInfinite(_("Infinite loop")),
     eSampleLoopPlayCount(_("Playback count"), 1),
     buttonSelectSample(UNICODE_LEFT_ARROW + "  " + _("Select Sample")),
+    editScriptSlotsButton(_("Edit Slots ...")),
+    dimregion(NULL),
     update_model(0)
 {
     // make synthesis parameter page tabs scrollable
     // (workaround for GTK3: default theme uses huge tabs which breaks layout)
     set_scrollable();
 
+    // use more appropriate increment/decrement steps for these spinboxes
+    eEG1PreAttack.set_increments(0.1, 5.0);
+    eEG2PreAttack.set_increments(0.1, 5.0);
+    eEG1Sustain.set_increments(0.1, 5.0);
+    eEG2Sustain.set_increments(0.1, 5.0);
+    eLFO1Frequency.set_increments(0.02, 0.2);
+    eLFO2Frequency.set_increments(0.02, 0.2);
+    eLFO3Frequency.set_increments(0.02, 0.2);
+    eLFO1Phase.set_increments(1.0, 10.0);
+    eLFO2Phase.set_increments(1.0, 10.0);
+    eLFO3Phase.set_increments(1.0, 10.0);
+
     connect(eEG1PreAttack, &gig::DimensionRegion::EG1PreAttack);
     connect(eEG1Attack, &gig::DimensionRegion::EG1Attack);
     connect(eEG1Decay1, &gig::DimensionRegion::EG1Decay1);
@@ -331,7 +485,9 @@ DimRegionEdit::DimRegionEdit() :
             &gig::eg_opt_t::Decay2Cancel);
     connect(eEG1StateOptions.checkBoxRelease, &gig::DimensionRegion::EG1Options,
             &gig::eg_opt_t::ReleaseCancel);
+    connect(eLFO1Wave, &gig::DimensionRegion::LFO1WaveForm);
     connect(eLFO1Frequency, &gig::DimensionRegion::LFO1Frequency);
+    connect(eLFO1Phase, &gig::DimensionRegion::LFO1Phase);
     connect(eLFO1InternalDepth, &gig::DimensionRegion::LFO1InternalDepth);
     connect(eLFO1ControlDepth, &gig::DimensionRegion::LFO1ControlDepth);
     connect(eLFO1Controller, &gig::DimensionRegion::LFO1Controller);
@@ -362,7 +518,9 @@ DimRegionEdit::DimRegionEdit() :
             &gig::eg_opt_t::Decay2Cancel);
     connect(eEG2StateOptions.checkBoxRelease, &gig::DimensionRegion::EG2Options,
             &gig::eg_opt_t::ReleaseCancel);
+    connect(eLFO2Wave, &gig::DimensionRegion::LFO2WaveForm);
     connect(eLFO2Frequency, &gig::DimensionRegion::LFO2Frequency);
+    connect(eLFO2Phase, &gig::DimensionRegion::LFO2Phase);
     connect(eLFO2InternalDepth, &gig::DimensionRegion::LFO2InternalDepth);
     connect(eLFO2ControlDepth, &gig::DimensionRegion::LFO2ControlDepth);
     connect(eLFO2Controller, &gig::DimensionRegion::LFO2Controller);
@@ -370,10 +528,13 @@ DimRegionEdit::DimRegionEdit() :
     connect(eLFO2Sync, &gig::DimensionRegion::LFO2Sync);
     connect(eEG3Attack, &gig::DimensionRegion::EG3Attack);
     connect(eEG3Depth, &gig::DimensionRegion::EG3Depth);
+    connect(eLFO3Wave, &gig::DimensionRegion::LFO3WaveForm);
     connect(eLFO3Frequency, &gig::DimensionRegion::LFO3Frequency);
+    connect(eLFO3Phase, &gig::DimensionRegion::LFO3Phase);
     connect(eLFO3InternalDepth, &gig::DimensionRegion::LFO3InternalDepth);
     connect(eLFO3ControlDepth, &gig::DimensionRegion::LFO3ControlDepth);
     connect(eLFO3Controller, &gig::DimensionRegion::LFO3Controller);
+    connect(eLFO3FlipPhase, &gig::DimensionRegion::LFO3FlipPhase);
     connect(eLFO3Sync, &gig::DimensionRegion::LFO3Sync);
     connect(eVCFEnabled, &gig::DimensionRegion::VCFEnabled);
     connect(eVCFType, &gig::DimensionRegion::VCFType);
@@ -427,7 +588,6 @@ DimRegionEdit::DimRegionEdit() :
     connect(eUnityNote, &DimRegionEdit::set_UnityNote);
     connect(eFineTune, &DimRegionEdit::set_FineTune);
     connect(eGain, &DimRegionEdit::set_Gain);
-    connect(eGainPlus6, &DimRegionEdit::set_Gain);
     connect(eSampleLoopEnabled, &DimRegionEdit::set_LoopEnabled);
     connect(eSampleLoopType, &DimRegionEdit::set_LoopType);
     connect(eSampleLoopStart, &DimRegionEdit::set_LoopStart);
@@ -438,7 +598,7 @@ DimRegionEdit::DimRegionEdit() :
         sigc::mem_fun(*this, &DimRegionEdit::onButtonSelectSamplePressed)
     );
 
-    for (int i = 0 ; i < 7 ; i++) {
+    for (int i = 0; i < tableSize; i++) {
 #if USE_GTKMM_GRID
         table[i] = new Gtk::Grid;
         table[i]->set_column_spacing(7);
@@ -543,6 +703,9 @@ DimRegionEdit::DimRegionEdit() :
     eLFO2FlipPhase.set_tip(
        "Inverts the LFO's generated wave vertically."
     );
+    eLFO3FlipPhase.set_tip(
+       "Inverts the LFO's generated wave vertically."
+    );
 
     pageno = 0;
     rowno = 0;
@@ -591,7 +754,6 @@ DimRegionEdit::DimRegionEdit() :
 
     addHeader(_("General Amplitude Settings"));
     addProp(eGain);
-    addProp(eGainPlus6);
     addProp(ePan);
     addHeader(_("Amplitude Envelope (EG1)"));
     addProp(eEG1PreAttack);
@@ -612,7 +774,9 @@ DimRegionEdit::DimRegionEdit() :
     nextPage();
 
     addHeader(_("Amplitude Oscillator (LFO1)"));
+    addProp(eLFO1Wave);
     addProp(eLFO1Frequency);
+    addProp(eLFO1Phase);
     addProp(eLFO1InternalDepth);
     addProp(eLFO1ControlDepth);
     {
@@ -630,6 +794,49 @@ DimRegionEdit::DimRegionEdit() :
     addProp(eLFO1Controller);
     addProp(eLFO1FlipPhase);
     addProp(eLFO1Sync);
+    {
+        Gtk::Frame* frame = new Gtk::Frame;
+        frame->add(lfo1Graph);
+        // on Gtk 3 there is no margin at all by default
+#if GTKMM_MAJOR_VERSION >= 3
+        frame->set_margin_top(12);
+        frame->set_margin_bottom(12);
+#endif
+#if USE_GTKMM_GRID
+        table[pageno]->attach(*frame, 1, rowno, 2);
+#else
+        table[pageno]->attach(*frame, 1, 3, rowno, rowno + 1,
+                              Gtk::SHRINK, Gtk::SHRINK);
+#endif
+        rowno++;
+    }
+    eLFO1Wave.signal_value_changed().connect(
+        sigc::mem_fun(lfo1Graph, &LFOGraph::queue_draw)
+    );
+    eLFO1Frequency.signal_value_changed().connect(
+        sigc::mem_fun(lfo1Graph, &LFOGraph::queue_draw)
+    );
+    eLFO1Phase.signal_value_changed().connect(
+        sigc::mem_fun(lfo1Graph, &LFOGraph::queue_draw)
+    );
+    eLFO1InternalDepth.signal_value_changed().connect(
+        sigc::mem_fun(lfo1Graph, &LFOGraph::queue_draw)
+    );
+    eLFO1ControlDepth.signal_value_changed().connect(
+        sigc::mem_fun(lfo1Graph, &LFOGraph::queue_draw)
+    );
+    eLFO1Controller.signal_value_changed().connect(
+        sigc::mem_fun(lfo1Graph, &LFOGraph::queue_draw)
+    );
+    eLFO1FlipPhase.signal_value_changed().connect(
+        sigc::mem_fun(lfo1Graph, &LFOGraph::queue_draw)
+    );
+    eLFO1Sync.signal_value_changed().connect(
+        sigc::mem_fun(lfo1Graph, &LFOGraph::queue_draw)
+    );
+
+    nextPage();
+
     addHeader(_("Crossfade"));
     addProp(eAttenuationController);
     addProp(eInvertAttenuationController);
@@ -668,14 +875,39 @@ DimRegionEdit::DimRegionEdit() :
     addHeader(_("General Filter Settings"));
     addProp(eVCFEnabled);
     {
-        const char* choices[] = { _("lowpass"), _("lowpassturbo"), _("bandpass"),
-                                  _("highpass"), _("bandreject"), 0 };
+        const char* choices[] = {
+            _("lowpass"), _("lowpassturbo"), _("bandpass"), _("highpass"),
+            _("bandreject"),
+            _("lowpass 1-pole [EXT]"),
+            _("lowpass 2-pole [EXT]"),
+            _("lowpass 4-pole [EXT]"),
+            _("lowpass 6-pole [EXT]"),
+            _("highpass 1-pole [EXT]"),
+            _("highpass 2-pole [EXT]"),
+            _("highpass 4-pole [EXT]"),
+            _("highpass 6-pole [EXT]"),
+            _("bandpass 2-pole [EXT]"),
+            _("bandreject 2-pole [EXT]"),
+            NULL
+        };
         static const gig::vcf_type_t values[] = {
+            // GigaStudio original filter types
             gig::vcf_type_lowpass,
             gig::vcf_type_lowpassturbo,
             gig::vcf_type_bandpass,
             gig::vcf_type_highpass,
-            gig::vcf_type_bandreject
+            gig::vcf_type_bandreject,
+            // LinuxSampler filter types (as gig format extension)
+            gig::vcf_type_lowpass_1p,
+            gig::vcf_type_lowpass_2p,
+            gig::vcf_type_lowpass_4p,
+            gig::vcf_type_lowpass_6p,
+            gig::vcf_type_highpass_1p,
+            gig::vcf_type_highpass_2p,
+            gig::vcf_type_highpass_4p,
+            gig::vcf_type_highpass_6p,
+            gig::vcf_type_bandpass_2p,
+            gig::vcf_type_bandreject_2p,
         };
         eVCFType.set_choices(choices, values);
     }
@@ -772,8 +1004,13 @@ DimRegionEdit::DimRegionEdit() :
     addProp(eEG2ControllerDecayInfluence);
     addProp(eEG2ControllerReleaseInfluence);
     addLine(eEG2StateOptions);
+
+    nextPage();
+
     lLFO2 = addHeader(_("Filter Cutoff Oscillator (LFO2)"));
+    addProp(eLFO2Wave);
     addProp(eLFO2Frequency);
+    addProp(eLFO2Phase);
     addProp(eLFO2InternalDepth);
     addProp(eLFO2ControlDepth);
     {
@@ -791,6 +1028,46 @@ DimRegionEdit::DimRegionEdit() :
     addProp(eLFO2Controller);
     addProp(eLFO2FlipPhase);
     addProp(eLFO2Sync);
+    {
+        Gtk::Frame* frame = new Gtk::Frame;
+        frame->add(lfo2Graph);
+        // on Gtk 3 there is no margin at all by default
+#if GTKMM_MAJOR_VERSION >= 3
+        frame->set_margin_top(12);
+        frame->set_margin_bottom(12);
+#endif
+#if USE_GTKMM_GRID
+        table[pageno]->attach(*frame, 1, rowno, 2);
+#else
+        table[pageno]->attach(*frame, 1, 3, rowno, rowno + 1,
+                              Gtk::SHRINK, Gtk::SHRINK);
+#endif
+        rowno++;
+    }
+    eLFO2Wave.signal_value_changed().connect(
+        sigc::mem_fun(lfo2Graph, &LFOGraph::queue_draw)
+    );
+    eLFO2Frequency.signal_value_changed().connect(
+        sigc::mem_fun(lfo2Graph, &LFOGraph::queue_draw)
+    );
+    eLFO2Phase.signal_value_changed().connect(
+        sigc::mem_fun(lfo2Graph, &LFOGraph::queue_draw)
+    );
+    eLFO2InternalDepth.signal_value_changed().connect(
+        sigc::mem_fun(lfo2Graph, &LFOGraph::queue_draw)
+    );
+    eLFO2ControlDepth.signal_value_changed().connect(
+        sigc::mem_fun(lfo2Graph, &LFOGraph::queue_draw)
+    );
+    eLFO2Controller.signal_value_changed().connect(
+        sigc::mem_fun(lfo2Graph, &LFOGraph::queue_draw)
+    );
+    eLFO2FlipPhase.signal_value_changed().connect(
+        sigc::mem_fun(lfo2Graph, &LFOGraph::queue_draw)
+    );
+    eLFO2Sync.signal_value_changed().connect(
+        sigc::mem_fun(lfo2Graph, &LFOGraph::queue_draw)
+    );
 
     nextPage();
 
@@ -801,7 +1078,9 @@ DimRegionEdit::DimRegionEdit() :
     addProp(eEG3Attack);
     addProp(eEG3Depth);
     addHeader(_("Pitch Oscillator (LFO3)"));
+    addProp(eLFO3Wave);
     addProp(eLFO3Frequency);
+    addProp(eLFO3Phase);
     addProp(eLFO3InternalDepth);
     addProp(eLFO3ControlDepth);
     {
@@ -817,7 +1096,48 @@ DimRegionEdit::DimRegionEdit() :
         eLFO3Controller.set_choices(choices, values);
     }
     addProp(eLFO3Controller);
+    addProp(eLFO3FlipPhase);
     addProp(eLFO3Sync);
+    {
+        Gtk::Frame* frame = new Gtk::Frame;
+        frame->add(lfo3Graph);
+        // on Gtk 3 there is no margin at all by default
+#if GTKMM_MAJOR_VERSION >= 3
+        frame->set_margin_top(12);
+        frame->set_margin_bottom(12);
+#endif
+#if USE_GTKMM_GRID
+        table[pageno]->attach(*frame, 1, rowno, 2);
+#else
+        table[pageno]->attach(*frame, 1, 3, rowno, rowno + 1,
+                              Gtk::SHRINK, Gtk::SHRINK);
+#endif
+        rowno++;
+    }
+    eLFO3Wave.signal_value_changed().connect(
+        sigc::mem_fun(lfo3Graph, &LFOGraph::queue_draw)
+    );
+    eLFO3Frequency.signal_value_changed().connect(
+        sigc::mem_fun(lfo3Graph, &LFOGraph::queue_draw)
+    );
+    eLFO3Phase.signal_value_changed().connect(
+        sigc::mem_fun(lfo3Graph, &LFOGraph::queue_draw)
+    );
+    eLFO3InternalDepth.signal_value_changed().connect(
+        sigc::mem_fun(lfo3Graph, &LFOGraph::queue_draw)
+    );
+    eLFO3ControlDepth.signal_value_changed().connect(
+        sigc::mem_fun(lfo3Graph, &LFOGraph::queue_draw)
+    );
+    eLFO3Controller.signal_value_changed().connect(
+        sigc::mem_fun(lfo3Graph, &LFOGraph::queue_draw)
+    );
+    eLFO3FlipPhase.signal_value_changed().connect(
+        sigc::mem_fun(lfo3Graph, &LFOGraph::queue_draw)
+    );
+    eLFO3Sync.signal_value_changed().connect(
+        sigc::mem_fun(lfo3Graph, &LFOGraph::queue_draw)
+    );
 
     nextPage();
 
@@ -970,13 +1290,54 @@ DimRegionEdit::DimRegionEdit() :
     eSampleLoopInfinite.signal_value_changed().connect(
         sigc::mem_fun(*this, &DimRegionEdit::loop_infinite_toggled));
 
+
+    addHeader(_("Script Patch Variables"));
+
+    m_labelPatchVarsDescr.set_markup(
+        _("These are variables declared in scripts with keyword "
+          "<span color='#FF4FF3'><b>patch</b></span>. "
+          "A 'patch' variable allows to override its default value on a per "
+          "instrument basis. That way a script may be shared by instruments "
+          "while being able to fine tune certain script parameters for each "
+          "instrument individually if necessary. Overridden default values "
+          "are displayed in bold. To revert back to the script's default "
+          "value, select the variable(s) and hit <b>&#x232b;</b> or "
+          "<b>&#x2326;</b> key.")
+    );
+    scriptVarsDescrBox.set_spacing(13);
+    scriptVarsDescrBox.pack_start(m_labelPatchVarsDescr, true, true);
+    scriptVarsDescrBox.pack_start(editScriptSlotsButton, false, false);
+#if USE_GTKMM_GRID
+    table[pageno]->attach(scriptVarsDescrBox, 1, rowno, 2);
+#else
+    table[pageno]->attach(scriptVarsDescrBox, 1, 3, rowno, rowno + 1,
+                          Gtk::EXPAND | Gtk::FILL, Gtk::SHRINK);
+#endif
+    rowno++;
+
+#if GTKMM_MAJOR_VERSION >= 3
+    scriptVars.set_margin_top(20);
+#endif
+#if USE_GTKMM_GRID
+    table[pageno]->attach(scriptVars, 1, rowno, 2);
+#else
+
+    table[pageno]->attach(scriptVars, 1, 3, rowno, rowno + 1,
+                          Gtk::EXPAND | Gtk::FILL, Gtk::EXPAND | Gtk::FILL);
+#endif
+    rowno++;
+
+
     append_page(*table[0], _("Sample"));
-    append_page(*table[1], _("Amplitude (1)"));
-    append_page(*table[2], _("Amplitude (2)"));
-    append_page(*table[3], _("Filter (1)"));
-    append_page(*table[4], _("Filter (2)"));
-    append_page(*table[5], _("Pitch"));
-    append_page(*table[6], _("Misc"));
+    append_page(*table[1], _("Amp (1)"));
+    append_page(*table[2], _("Amp (2)"));
+    append_page(*table[3], _("Amp (3)"));
+    append_page(*table[4], _("Filter (1)"));
+    append_page(*table[5], _("Filter (2)"));
+    append_page(*table[6], _("Filter (3)"));
+    append_page(*table[7], _("Pitch"));
+    append_page(*table[8], _("Misc"));
+    append_page(*table[9], _("Script"));
 
     Settings::singleton()->showTooltips.get_proxy().signal_changed().connect(
         sigc::mem_fun(*this, &DimRegionEdit::on_show_tooltips_changed)
@@ -1131,17 +1492,6 @@ void DimRegionEdit::addProp(BoolEntry& boolentry)
     rowno++;
 }
 
-void DimRegionEdit::addProp(BoolEntryPlus6& boolentry)
-{
-#if USE_GTKMM_GRID
-    table[pageno]->attach(boolentry.widget, 1, rowno, 2);
-#else
-    table[pageno]->attach(boolentry.widget, 1, 3, rowno, rowno + 1,
-                          Gtk::FILL, Gtk::SHRINK);
-#endif
-    rowno++;
-}
-
 void DimRegionEdit::addProp(LabelWidget& prop)
 {
 #if USE_GTKMM_GRID
@@ -1185,6 +1535,9 @@ void DimRegionEdit::set_dim_region(gig::DimensionRegion* d)
     release_curve.set_dim_region(d);
     cutoff_curve.set_dim_region(d);
     crossfade_curve.set_dim_region(d);
+    lfo1Graph.set_dim_region(d);
+    lfo2Graph.set_dim_region(d);
+    lfo3Graph.set_dim_region(d);
 
     set_sensitive(d);
     if (!d) return;
@@ -1208,7 +1561,9 @@ void DimRegionEdit::set_dim_region(gig::DimensionRegion* d)
     eEG1StateOptions.checkBoxDecay1.set_value(d->EG1Options.Decay1Cancel);
     eEG1StateOptions.checkBoxDecay2.set_value(d->EG1Options.Decay2Cancel);
     eEG1StateOptions.checkBoxRelease.set_value(d->EG1Options.ReleaseCancel);
+    eLFO1Wave.set_value(d->LFO1WaveForm);
     eLFO1Frequency.set_value(d->LFO1Frequency);
+    eLFO1Phase.set_value(d->LFO1Phase);
     eLFO1InternalDepth.set_value(d->LFO1InternalDepth);
     eLFO1ControlDepth.set_value(d->LFO1ControlDepth);
     eLFO1Controller.set_value(d->LFO1Controller);
@@ -1231,7 +1586,9 @@ void DimRegionEdit::set_dim_region(gig::DimensionRegion* d)
     eEG2StateOptions.checkBoxDecay1.set_value(d->EG2Options.Decay1Cancel);
     eEG2StateOptions.checkBoxDecay2.set_value(d->EG2Options.Decay2Cancel);
     eEG2StateOptions.checkBoxRelease.set_value(d->EG2Options.ReleaseCancel);
+    eLFO2Wave.set_value(d->LFO2WaveForm);
     eLFO2Frequency.set_value(d->LFO2Frequency);
+    eLFO2Phase.set_value(d->LFO2Phase);
     eLFO2InternalDepth.set_value(d->LFO2InternalDepth);
     eLFO2ControlDepth.set_value(d->LFO2ControlDepth);
     eLFO2Controller.set_value(d->LFO2Controller);
@@ -1239,10 +1596,13 @@ void DimRegionEdit::set_dim_region(gig::DimensionRegion* d)
     eLFO2Sync.set_value(d->LFO2Sync);
     eEG3Attack.set_value(d->EG3Attack);
     eEG3Depth.set_value(d->EG3Depth);
+    eLFO3Wave.set_value(d->LFO3WaveForm);
     eLFO3Frequency.set_value(d->LFO3Frequency);
+    eLFO3Phase.set_value(d->LFO3Phase);
     eLFO3InternalDepth.set_value(d->LFO3InternalDepth);
     eLFO3ControlDepth.set_value(d->LFO3ControlDepth);
     eLFO3Controller.set_value(d->LFO3Controller);
+    eLFO3FlipPhase.set_value(d->LFO3FlipPhase);
     eLFO3Sync.set_value(d->LFO3Sync);
     eVCFEnabled.set_value(d->VCFEnabled);
     eVCFType.set_value(d->VCFType);
@@ -1332,7 +1692,6 @@ void DimRegionEdit::set_dim_region(gig::DimensionRegion* d)
     buttonSelectSample.set_sensitive(d && d->pSample);
     eFineTune.set_value(d->FineTune);
     eGain.set_value(d->Gain);
-    eGainPlus6.set_value(d->Gain);
     eSampleLoopEnabled.set_value(d->SampleLoops);
     eSampleLoopType.set_value(
         d->SampleLoops ? d->pSampleLoops[0].LoopType : 0);
@@ -1349,6 +1708,10 @@ void DimRegionEdit::set_dim_region(gig::DimensionRegion* d)
     wSample->set_text(d->pSample ? gig_to_utf8(d->pSample->pInfo->Name) :
                       _("NULL"));
 
+    scriptVars.setInstrument(
+        (gig::Instrument*) d->GetParent()->GetParent()
+    );
+
     update_loop_elements();
     VCFEnabled_toggled();
 }
@@ -1380,12 +1743,15 @@ void DimRegionEdit::VCFEnabled_toggled()
     eEG2ControllerReleaseInfluence.set_sensitive(sensitive);
     eEG2StateOptions.set_sensitive(sensitive);
     lLFO2->set_sensitive(sensitive);
+    eLFO2Wave.set_sensitive(sensitive);
     eLFO2Frequency.set_sensitive(sensitive);
+    eLFO2Phase.set_sensitive(sensitive);
     eLFO2InternalDepth.set_sensitive(sensitive);
     eLFO2ControlDepth.set_sensitive(sensitive);
     eLFO2Controller.set_sensitive(sensitive);
     eLFO2FlipPhase.set_sensitive(sensitive);
     eLFO2Sync.set_sensitive(sensitive);
+    lfo2Graph.set_sensitive(sensitive);
     if (sensitive) {
         VCFCutoffController_changed();
         VCFResonanceController_changed();
@@ -1576,6 +1942,8 @@ bool DimRegionEdit::set_sample(gig::Sample* sample, bool copy_sample_unity, bool
     for (std::set<gig::DimensionRegion*>::iterator itDimReg = dimregs.begin();
          itDimReg != dimregs.end(); ++itDimReg)
     {
+        //NOTE: sample may be NULL !
+        // (exactly: if called by nullOutSampleReference())
         result |= set_sample(*itDimReg, sample, copy_sample_unity, copy_sample_tune, copy_sample_loop);
     }
     return result;
@@ -1586,14 +1954,21 @@ bool DimRegionEdit::set_sample(gig::DimensionRegion* dimreg, gig::Sample* sample
     if (dimreg) {
         //TODO: we should better move the code from MainWindow::on_sample_label_drop_drag_data_received() here
 
+        //NOTE: sample may be NULL !
+        // (exactly: if called by nullOutSampleReference())
+
         // currently commented because we're sending a similar signal in MainWindow::on_sample_label_drop_drag_data_received()
         //DimRegionChangeGuard(this, dimregion);
 
+        gig::Sample* oldref = dimreg->pSample;
+
         // make sure stereo samples always are the same in both
         // dimregs in the samplechannel dimension
         int nbDimregs = 1;
         gig::DimensionRegion* d[2] = { dimreg, 0 };
-        if (sample->Channels == 2) {
+        if ((sample && sample->Channels == 2) ||
+            (!sample && oldref && oldref->Channels == 2))
+        {
             gig::Region* region = dimreg->GetParent();
 
             int bitcount = 0;
@@ -1619,8 +1994,6 @@ bool DimRegionEdit::set_sample(gig::DimensionRegion* dimreg, gig::Sample* sample
             }
         }
 
-        gig::Sample* oldref = dimreg->pSample;
-
         for (int i = 0 ; i < nbDimregs ; i++) {
             d[i]->pSample = sample;
 
@@ -1649,7 +2022,10 @@ bool DimRegionEdit::set_sample(gig::DimensionRegion* dimreg, gig::Sample* sample
 
         // update ui
         update_model++;
-        wSample->set_text(gig_to_utf8(dimreg->pSample->pInfo->Name));
+        wSample->set_text(
+            dimreg->pSample ? gig_to_utf8(dimreg->pSample->pInfo->Name) :
+                              _("NULL")
+        );
         eUnityNote.set_value(dimreg->UnityNote);
         eFineTune.set_value(dimreg->FineTune);
         eSampleLoopEnabled.set_value(dimreg->SampleLoops);
@@ -1789,47 +2165,7 @@ void DimRegionEdit::set_LoopPlayCount(gig::DimensionRegion& d, uint32_t value)
 
 void DimRegionEdit::nullOutSampleReference() {
     if (!dimregion) return;
-    gig::Sample* oldref = dimregion->pSample;
-    if (!oldref) return;
-
-    DimRegionChangeGuard(this, dimregion);
-
-    // in case currently assigned sample is a stereo one, then remove both
-    // references (expected to be due to a "stereo dimension")
-    gig::DimensionRegion* d[2] = { dimregion, NULL };
-    if (oldref->Channels == 2) {
-        gig::Region* region = dimregion->GetParent();
-        {
-            int stereo_bit = 0;
-            int bitcount = 0;
-            for (int dim = 0 ; dim < region->Dimensions ; dim++) {
-                if (region->pDimensionDefinitions[dim].dimension == gig::dimension_samplechannel) {
-                    stereo_bit = 1 << bitcount;
-                    break;
-                }
-                bitcount += region->pDimensionDefinitions[dim].bits;
-            }
-
-            if (stereo_bit) {
-                int dimregno;
-                for (dimregno = 0 ; dimregno < region->DimensionRegions ; dimregno++) {
-                    if (region->pDimensionRegions[dimregno] == dimregion) {
-                        break;
-                    }
-                }
-                d[0] = region->pDimensionRegions[dimregno & ~stereo_bit];
-                d[1] = region->pDimensionRegions[dimregno | stereo_bit];
-            }
-        }
-    }
-
-    if (d[0]) d[0]->pSample = NULL;
-    if (d[1]) d[1]->pSample = NULL;
-
-    // update UI elements
-    set_dim_region(dimregion);
-
-    sample_ref_changed_signal.emit(oldref, NULL);
+    set_sample(NULL/*sample*/, false, false, false);
 }
 
 void DimRegionEdit::onButtonSelectSamplePressed() {
